@@ -2,12 +2,14 @@ package SVN::Web::action;
 
 =head1 NAME
 
-SVN::Web::action - documentation for writing new SVN::Web actions
+SVN::Web::action - base class for SVN::Web::actions
 
 =head1 DESCRIPTION
 
-This file contains no code.  It provides documentation for writing new
-SVN::Web actions.
+This is the base class for all SVN::Web actions.  It provides a
+constructor and some useful utility methods that actions may find
+useful.  It also contains documentation for anyone interested in
+writing new SVN::Web actions.
 
 =head1 OVERVIEW
 
@@ -30,24 +32,27 @@ for that action.
         option2: value2
     ...
 
-Each action is a class that must implement at least two methods,
-C<new()> and C<run()>.
+Each action is a class that must implement a C<run()> method.
+
+=cut
+
+sub new {
+    my $class = shift;
+    my $self = bless {}, $class;
+    %$self = @_;
+
+    return $self;
+}
+
+=head1 SUBCLASSING
+
+Actions should derive from L<SVN::Web::action>.  This gives them a
+default constructor that generates a hash based object.
+
+  use base 'SVN::Web::action';
 
 =head1 METHODS
 
-=head2 new()
-
-This is a traditional Perl constructor.  The following boilerplate
-code will suffice.
-
-  sub new {
-      my $class = shift;
-      my $self = bless {}, $class;
-      %$self = @_;
-
-      return $self;
-  }
-
 =head2 run()
 
 The C<run> method is where the action carries out its work.
@@ -195,6 +200,105 @@ scalar that contains the HTML to be sent to the browser.
 
   return "<p>hello, world</p>";
 
+=head1 UTILITY METHODS
+
+The following methods are intended to share common code among actions.
+
+=head2 recent_interesting_rev($path, $rev)
+
+Given a repository path, and a revision number, returns the most recent
+interesting revision for the path that is the same as, or older (i.e.,
+smaller) than the revision number.
+
+=cut
+
+sub recent_interesting_rev {
+    my $self = shift;
+    my $path = shift;
+    my $rev  = shift;
+
+    my $fs   = $self->{repos}->fs;
+    my $root = $fs->revision_root($rev);
+
+    my $hist = $root->node_history($path);
+    $hist    = $hist->prev(0);
+    $rev     = ($hist->location())[1];
+
+    return $rev;
+}
+
+=head2 get_revs()
+
+Returns a list of 4 items.  In order, they are:
+
+=over
+
+=item Explicit rev
+
+The value of any CGI C<rev> parameter passed to the action ($exp_rev).
+
+=item Youngest rev
+
+The repository's youngest revision ($yng_rev) for the current path.
+This is not necessarily the same as the repositories youngest
+revision.
+
+=item Actual rev
+
+The actual revision ($act_rev) that will be acted on.  This is the
+explicit rev, if it's defined, otherwise it's the youngest rev.
+
+=item Head
+
+A boolean value indicating whether or not we can be considered to be
+at the HEAD of the repository ($at_head).
+
+=back
+
+=cut
+
+sub get_revs {
+    my $self = shift;
+    my $path = $self->{path};
+    my $fs   = $self->{repos}->fs();
+
+    my $exp_rev = $self->{cgi}->param('rev');
+    my $yng_rev = $fs->youngest_rev();
+    my $act_rev = defined $exp_rev ? $self->recent_interesting_rev($path, $exp_rev) :
+                                     $self->recent_interesting_rev($path, $yng_rev);
+
+    my $at_head = 0;
+    if(! defined $exp_rev) {
+	$at_head = 1;
+    } else {
+	if($exp_rev == $yng_rev) {
+	    $at_head = 1;
+	}
+    }
+    return($exp_rev, $yng_rev, $act_rev, $at_head);
+}
+
+=head1 CACHING
+
+If the output from the action can usefully be cached then consider
+implementing a C<cache_key> method.
+
+This method receives the same parameters as the C<run()> method, and
+must use those parameters to generate a unique key for the content
+generated by the C<run()> method.
+
+For example, consider the standard C<Revision> action.  This action
+only depends on a single parameter -- the repository revision number.
+So that makes a good cache key.
+
+  sub cache_key {
+      my $self = shift;
+
+      return $self->{cgi}->param('rev');
+  }
+
+Other actions may have more complicated keys.
+
 =head1 ERRORS AND EXCEPTIONS
 
 If your action needs to fail for some reason -- perhaps the parameters
@@ -205,4 +309,24 @@ Exceptions, along with examples, are described in L<SVN::Web::X>.
 
 =cut
 
+# Diff.pm and Revision.pm need to munge the output in Text::Diff::HTML in
+# the same way.  The code lives here for the time being, although it's not
+# probably not the optimal place to put it.
+
+sub _munge_html_diff {
+    my $self = shift;
+    my $html = shift;
+
+    $html =~ s/^  /<span class="diff-leader">  <\/span>/mg;
+    $html =~ s/<span class="ctx">  /<span class="ctx"><span class="diff-leader">  <\/span>/mg;
+    $html =~ s/<ins>\+ /<span class="ins"><span class="diff-leader">+ <\/span>/mg;
+    $html =~ s/<del>- /<span class="del"><span class="diff-leader">- <\/span>/mg;
+    $html =~ s/<\/ins>/<\/span>/mg;
+    $html =~ s/<\/del>/<\/span>/mg;
+    $html =~ s/^- /<span class="diff-leader">- <\/span>/mg;
+    $html =~ s/^\+ /<span class="diff-leader">+ <\/span>/mg;
+
+    return $html;
+}
+
 1;
